@@ -99,7 +99,7 @@ fn get_corresponding_type(elements: &[(Option<TypeStruct>, Vec<String>)],
                 file_comment = true;
             } else {
                 while line > 0 && (line + *decal) > 0 &&
-                      original_content[line + *decal - 1].trim_left().starts_with("#") {
+                      original_content[line + *decal - 1].trim_start().starts_with("#") {
                     line -= 1;
                 }
             }
@@ -252,14 +252,7 @@ fn do_regenerate(path: &Path, parse_result: &mut ParseResult,
                 }
             }
             EventType::InScope => {
-                current = {
-                    let t = strip::add_to_type_scope(&current, &waiting_type);
-                    if ignore_macros {
-                        erase_macro_path(t)
-                    } else {
-                        t
-                    }
-                };
+                current = strip::add_to_type_scope(&current, &waiting_type);
                 waiting_type = None;
             }
             EventType::OutScope => {
@@ -378,19 +371,19 @@ fn sub_erase_macro_path(ty: Option<Box<TypeStruct>>, is_parent: bool) -> Option<
                 tmp.parent = sub_erase_macro_path(t.clone().parent, true);
                 Some(tmp)
             }
-        },
+        }
         Some(t) => {
             let mut tmp = t.clone();
             tmp.parent = sub_erase_macro_path(t.parent, true);
             Some(tmp)
-        },
+        }
         None => None,
     }
 }
 
 fn erase_macro_path(ty: Option<TypeStruct>) -> Option<TypeStruct> {
     if let Some(t) = ty {
-        Some((*sub_erase_macro_path(Some(Box::new(t)), false).unwrap()))
+        Some(*sub_erase_macro_path(Some(Box::new(t)), false).unwrap())
     } else {
         None
     }
@@ -478,10 +471,10 @@ where S: Deref<Target = str>,
                     comments.push(line[..].to_owned());
                 }
                 State::File {
-                    file: file,
-                    infos: infos,
+                    file,
+                    infos,
                     ty: if ignore_macros { erase_macro_path(ty) } else { ty },
-                    comments: comments,
+                    comments,
                 }
             },
         }
